@@ -1,10 +1,10 @@
 package Test::Perinci::CmdLine;
 
-our $DATE = '2016-08-30'; # DATE
-our $VERSION = '1.41'; # VERSION
+our $DATE = '2016-10-29'; # DATE
+our $VERSION = '1.42'; # VERSION
 
 use 5.010001;
-use strict;
+use strict 'subs', 'vars';
 use warnings;
 use Devel::Confess;
 
@@ -19,7 +19,11 @@ use Test::More 0.98;
 
 require Exporter;
 our @ISA = qw(Exporter);
-our @EXPORT = qw(pericmd_ok);
+our @EXPORT = (
+    'pericmd_ok', # old, back-compat
+    'pericmd_run_suite_ok',
+    'pericmd_run_ok',
+);
 
 our %SPEC;
 
@@ -33,6 +37,9 @@ my %common_args = (
         ]],
         req => 1,
     },
+);
+
+my %incl_excl_tags_args = (
     include_tags => {
         schema => ['array*', of=>'str*'],
     },
@@ -41,11 +48,159 @@ my %common_args = (
     },
 );
 
-$SPEC{run_test_groups} = {
+my %run_args = (
+    name => {
+        summary => 'Test name',
+        description => <<'_',
+
+If not specified, a nice default will be picked (e.g. from `argv`).
+
+_
+        schema => 'str*',
+    },
+    gen_args => {
+        summary => 'Arguments to be passed to `Perinci::CmdLine->new()`',
+        schema => 'hash*',
+        req => 1,
+    },
+    inline_gen_args => {
+        summary => 'Additional arguments to be passed to '.
+            '`Perinci::CmdLine::Inline->new()`',
+        description => <<'_',
+
+Keys from this argument will be added to `gen_args` and will only be used when
+`class` is `Perinci::CmdLine::Inline`.
+
+_
+        schema => 'hash*',
+    },
+    classic_gen_args => {
+        summary => 'Additional arguments to be passed to '.
+            '`Perinci::CmdLine::Classic->new()`',
+        description => <<'_',
+
+Keys from this argument will be added to `gen_args` and will only be used when
+`class` is `Perinci::CmdLine::Classic`.
+
+_
+        schema => 'hash*',
+    },
+    lite_gen_args => {
+        summary => 'Additional arguments to be passed to '.
+            '`Perinci::CmdLine::Lite->new()`',
+        description => <<'_',
+
+Keys from this argument will be added to `gen_args` and will only be used when
+`class` is `Perinci::CmdLine::Lite`.
+
+_
+        schema => 'hash*',
+    },
+    argv => {
+        summary => 'Command-line arguments that will be passed to '.
+            'generated CLI script',
+        schema => 'array*',
+        default => [],
+    },
+    stdin => {
+        summary => "Supply stdin content to generated CLI script",
+        schema => 'str*',
+    },
+    env => {
+        summary => "Set environment variables for generated CLI script",
+        schema => 'hash*',
+    },
+    comp_line0 => {
+        summary => "Set COMP_LINE environment for generated CLI script",
+        description => <<'_',
+
+Can contain `^` (caret) character which will be stripped from the final
+`COMP_LINE` and the position of the character will be used to determine
+`COMP_POINT`.
+
+_
+        schema => 'str*',
+    },
+    inline_allow => {
+        summary => "Modules to allow to be loaded when testing generated ".
+            "Perinci::CmdLine::Inline script",
+        description => <<'_',
+
+By default, when running the generated Perinci::CmdLine::Inline script, this
+perl option will be used (see <pm:lib::filter> for more details):
+
+    -Mlib::filter=allow_noncore,0
+
+This means the script will only be able to load core modules. But if the script
+is allowed to load additional modules, you can set this `inline_allow` parameter
+to, e.g. `["Foo::Bar","Baz"]` and the above perl option will become:
+
+    -Mlib::filter=allow_noncore,0,allow=Foo::Bar;Baz
+
+_
+        schema => ['array*', of=>'perl::modname*'],
+    },
+
+    exit_code => {
+        summary => "Expected script's exit code",
+        schema => 'int*',
+        default => 0,
+        tags => ['category:assert'],
+    },
+    exit_code_like => {
+        summary => "Expected script's exit code (as regex pattern)",
+        schema => 're*',
+        default => 0,
+        tags => ['category:assert'],
+    },
+    stdout_like => {
+        summary => "Test output of generated CLI script",
+        schema => 're*',
+        tags => ['category:assert'],
+    },
+    stdout_unlike => {
+        summary => "Test output of generated CLI script",
+        schema => 're*',
+        tags => ['category:assert'],
+    },
+    stderr_like => {
+        summary => "Test error output of generated CLI script",
+        schema => 're*',
+        tags => ['category:assert'],
+    },
+    stderr_unlike => {
+        summary => "Test error output of generated CLI script",
+        schema => 're*',
+        tags => ['category:assert'],
+    },
+    comp_answer => {
+        summary => "Test completion answer of generated CLI script",
+        schema => ['array*', of=>'str*'],
+        tags => ['category:assert'],
+    },
+    posttest => {
+        summary => "Additional tests",
+        description => <<'_',
+
+For example you can do `is()` or `ok()` or other <pm:Test::More> tests.
+
+_
+        schema => 'code*',
+        tags => ['category:assert'],
+    },
+
+    tags => {
+        schema => 'array*',
+        tags => ['hidden'],
+    },
+);
+
+$SPEC{pericmd_run_test_groups_ok} = {
     v => 1.1,
     summary => 'Run groups of Perinci::CmdLine tests',
     args => {
         %common_args,
+        %incl_excl_tags_args,
         tempdir => {
             schema => 'str*',
             description => <<'_',
@@ -64,7 +219,7 @@ _
         },
     },
 };
-sub run_test_groups {
+sub pericmd_run_test_groups_ok {
     my %args = @_;
 
     my $class   = $args{class};
@@ -121,6 +276,8 @@ sub run_test_groups {
             if ($test_args{gen_args}) {
                 $gen_args{$_} = $test_args{gen_args}{$_}
                     for keys %{$test_args{gen_args}};
+            } else {
+                die "Please specify 'gen_args'";
             }
             if ($class eq 'Perinci::CmdLine::Lite' &&
                     $test_args{lite_gen_args}) {
@@ -230,7 +387,7 @@ sub run_test_groups {
         my %test_args = @_;
 
         my $comp_line = delete($test_args{comp_line0});
-        my $answer = delete($test_args{answer});
+        my $answer = delete($test_args{comp_answer});
 
         my $comp_point;
         if (($comp_point = index($comp_line, '^')) >= 0) {
@@ -303,14 +460,48 @@ sub run_test_groups {
     }
 }
 
-$SPEC{pericmd_ok} = {
+$SPEC{pericmd_run_ok} = {
+    v => 1.1,
+    summary => 'Run a single test of a Perinci::CmdLine script',
+    args => {
+        %common_args,
+        %run_args,
+    },
+};
+sub pericmd_run_ok {
+    my %args = @_;
+
+    my %rtg_args;
+
+    $rtg_args{class} = delete $args{class};
+
+    {
+        my $test = {};
+        for my $k (keys %run_args) {
+            $test->{$k} = delete $args{$k} if exists $args{$k};
+        }
+        my $group = {
+            name => $test->{name} // 'single test group (pericmd_run_ok)',
+        };
+        if (defined $args{comp_answer}) {
+            $group->{completion_tests} = [$test];
+        } else {
+            $group->{tests} = [$test];
+        }
+        $rtg_args{groups} = [$group];
+    }
+
+    pericmd_run_test_groups_ok(%rtg_args);
+}
+
+$SPEC{pericmd_run_suite_ok} = {
     v => 1.1,
     summary => 'Common test suite for Perinci::CmdLine::{Lite,Classic,Inline}',
     args => {
         %common_args,
     },
 };
-sub pericmd_ok {
+sub pericmd_run_suite_ok {
     my %suite_args = @_;
 
     my $tempdir = tempdir();
@@ -339,7 +530,7 @@ $SPEC{square} = {v=>1.1, args=>{num=>{schema=>'num*', req=>1, pos=>0}}};
 sub square { my %args=@_; [200, "OK", $args{num}**2] }
 !;
 
-    run_test_groups(
+    pericmd_run_test_groups_ok(
         %suite_args,
         include_tags => $include_tags,
         exclude_tags => $exclude_tags,
@@ -1165,7 +1356,7 @@ sub square { my %args=@_; [200, "OK", $args{num}**2] }
                         inline_gen_args => {load_module=>['Perinci::Examples::Tiny']},
                         argv           => [],
                         comp_line0     => 'cmd --nu^',
-                        answer         => ['--number'],
+                        comp_answer    => ['--number'],
                     },
                     {
                         tags           => ['subcommand'],
@@ -1180,7 +1371,7 @@ sub square { my %args=@_; [200, "OK", $args{num}**2] }
                         inline_gen_args => {load_module=>['Perinci::Examples::Tiny']},
                         argv           => [],
                         comp_line0     => 'cmd sc^',
-                        answer         => ['sc1', 'sc2'],
+                        comp_answer    => ['sc1', 'sc2'],
                     },
                     {
                         tags           => ['subcommand'],
@@ -1195,7 +1386,7 @@ sub square { my %args=@_; [200, "OK", $args{num}**2] }
                         inline_gen_args => {load_module=>['Perinci::Examples::Tiny']},
                         argv           => [],
                         comp_line0     => 'cmd sc2 --nu^',
-                        answer         => ['--number'],
+                        comp_answer    => ['--number'],
                     },
                 ],
             }, # completion
@@ -1503,6 +1694,9 @@ _
     );
 }
 
+# old, back-compat name
+*pericmd_ok = \&pericmd_run_suite_ok;
+
 1;
 # ABSTRACT: Common test suite for Perinci::CmdLine::{Lite,Classic,Inline}
 
@@ -1518,14 +1712,16 @@ Test::Perinci::CmdLine - Common test suite for Perinci::CmdLine::{Lite,Classic,I
 
 =head1 VERSION
 
-This document describes version 1.41 of Test::Perinci::CmdLine (from Perl distribution Test-Perinci-CmdLine), released on 2016-08-30.
+This document describes version 1.42 of Test::Perinci::CmdLine (from Perl distribution Test-Perinci-CmdLine), released on 2016-10-29.
+
+=for Pod::Coverage ^(pericmd_ok)$
 
 =head1 FUNCTIONS
 
 
-=head2 pericmd_ok(%args) -> [status, msg, result, meta]
+=head2 pericmd_run_ok(%args) -> [status, msg, result, meta]
 
-Common test suite for Perinci::CmdLine::{Lite,Classic,Inline}.
+Run a single test of a Perinci::CmdLine script.
 
 This function is exported by default.
 
@@ -1533,13 +1729,109 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
+=item * B<argv> => I<array> (default: [])
+
+Command-line arguments that will be passed to generated CLI script.
+
 =item * B<class>* => I<str>
 
 Which Perinci::CmdLine class are we testing.
 
-=item * B<exclude_tags> => I<array[str]>
+=item * B<classic_gen_args> => I<hash>
 
-=item * B<include_tags> => I<array[str]>
+Additional arguments to be passed to `Perinci::CmdLine::Classic->new()`.
+
+Keys from this argument will be added to C<gen_args> and will only be used when
+C<class> is C<Perinci::CmdLine::Classic>.
+
+=item * B<comp_answer> => I<array[str]>
+
+Test completion answer of generated CLI script.
+
+=item * B<comp_line0> => I<str>
+
+Set COMP_LINE environment for generated CLI script.
+
+Can contain C<^> (caret) character which will be stripped from the final
+C<COMP_LINE> and the position of the character will be used to determine
+C<COMP_POINT>.
+
+=item * B<env> => I<hash>
+
+Set environment variables for generated CLI script.
+
+=item * B<exit_code> => I<int> (default: 0)
+
+Expected script's exit code.
+
+=item * B<exit_code_like> => I<re> (default: 0)
+
+Expected script's exit code (as regex pattern).
+
+=item * B<gen_args>* => I<hash>
+
+Arguments to be passed to `Perinci::CmdLine->new()`.
+
+=item * B<inline_allow> => I<array[perl::modname]>
+
+Modules to allow to be loaded when testing generated Perinci::CmdLine::Inline script.
+
+By default, when running the generated Perinci::CmdLine::Inline script, this
+perl option will be used (see L<lib::filter> for more details):
+
+ -Mlib::filter=allow_noncore,0
+
+This means the script will only be able to load core modules. But if the script
+is allowed to load additional modules, you can set this C<inline_allow> parameter
+to, e.g. C<["Foo::Bar","Baz"]> and the above perl option will become:
+
+ -Mlib::filter=allow_noncore,0,allow=Foo::Bar;Baz
+
+=item * B<inline_gen_args> => I<hash>
+
+Additional arguments to be passed to `Perinci::CmdLine::Inline->new()`.
+
+Keys from this argument will be added to C<gen_args> and will only be used when
+C<class> is C<Perinci::CmdLine::Inline>.
+
+=item * B<lite_gen_args> => I<hash>
+
+Additional arguments to be passed to `Perinci::CmdLine::Lite->new()`.
+
+Keys from this argument will be added to C<gen_args> and will only be used when
+C<class> is C<Perinci::CmdLine::Lite>.
+
+=item * B<name> => I<str>
+
+Test name.
+
+If not specified, a nice default will be picked (e.g. from C<argv>).
+
+=item * B<posttest> => I<code>
+
+Additional tests.
+
+For example you can do C<is()> or C<ok()> or other L<Test::More> tests.
+
+=item * B<stderr_like> => I<re>
+
+Test error output of generated CLI script.
+
+=item * B<stderr_unlike> => I<re>
+
+Test error output of generated CLI script.
+
+=item * B<stdin> => I<str>
+
+Supply stdin content to generated CLI script.
+
+=item * B<stdout_like> => I<re>
+
+Test output of generated CLI script.
+
+=item * B<stdout_unlike> => I<re>
+
+Test output of generated CLI script.
 
 =back
 
@@ -1555,7 +1847,35 @@ that contains extra information.
 Return value:  (any)
 
 
-=head2 run_test_groups(%args) -> [status, msg, result, meta]
+=head2 pericmd_run_suite_ok(%args) -> [status, msg, result, meta]
+
+Common test suite for Perinci::CmdLine::{Lite,Classic,Inline}.
+
+This function is exported by default.
+
+Arguments ('*' denotes required arguments):
+
+=over 4
+
+=item * B<class>* => I<str>
+
+Which Perinci::CmdLine class are we testing.
+
+=back
+
+Returns an enveloped result (an array).
+
+First element (status) is an integer containing HTTP status code
+(200 means OK, 4xx caller error, 5xx function error). Second element
+(msg) is a string containing error message, or 'OK' if status is
+200. Third element (result) is optional, the actual result. Fourth
+element (meta) is called result metadata and is optional, a hash
+that contains extra information.
+
+Return value:  (any)
+
+
+=head2 pericmd_run_test_groups_ok(%args) -> [status, msg, result, meta]
 
 Run groups of Perinci::CmdLine tests.
 
